/*!---------------------------------------------------------
* Copyright (C) Microsoft Corporation. All rights reserved.
*--------------------------------------------------------*/
/// <reference path="./declare/node.d.ts" />
/// <reference path="./declare/atom-browser.d.ts" />
'use strict';
var crypto = require('crypto');
var fs = require('fs');
var path = require('path');
var os = require('os');
var app = require('app');
var arrays = require('./arrays');
var objects = require('./objects');
var paths = require('./paths');
var packageJson = require('./package.json');
exports.isBuiltTicino = fs.existsSync(path.join(__dirname, 'client'));
exports.flavor = packageJson.flavor || void 0;
exports.commit = packageJson.commit || void 0;
exports.updateInfo = packageJson.update || void 0;
// !!! Do not move this down, it needs to run early !!!
var appData = app.getPath('appData');
if (!exports.isBuiltTicino) {
    app.setPath('userData', path.join(appData, 'Ticino-Development'));
}
else if (exports.flavor) {
    app.setPath('userData', path.join(appData, exports.flavor));
}
// !!!
exports.appNameShort = 'Code';
exports.appNameLong = 'Visual Studio Code';
exports.isWindows = (process.platform === 'win32');
exports.isMac = (process.platform === 'darwin');
exports.isLinux = (process.platform === 'linux');
exports.runningInstanceHandle = getRunningInstanceHandle();
exports.appRoot = getAppRoot();
exports.version = app.getVersion();
exports.cliArgs = parseCli();
exports.appHome = app.getPath('userData');
exports.appSettingsHome = path.join(exports.appHome, 'User');
if (!fs.existsSync(exports.appSettingsHome)) {
    fs.mkdirSync(exports.appSettingsHome);
}
exports.appSettingsPath = path.join(exports.appSettingsHome, 'settings.json');
exports.appKeybindingsPath = path.join(exports.appSettingsHome, 'keybindings.json');
exports.appLaunchSettingsPath = path.join(exports.appSettingsHome, 'launch.json');
exports.welcomePath = path.join(__dirname, 'welcome-mac.md');
function log() {
    var a = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        a[_i - 0] = arguments[_i];
    }
    if (exports.cliArgs.verboseLogging) {
        console.log.apply(null, a);
    }
}
exports.log = log;
function parseCli() {
    // We need to do some argv massaging. First, remove the Atom executable
    var args = Array.prototype.slice.call(process.argv, 1);
    // Then, when in dev, remove the first non option argument, it will be the app location
    if (!exports.isBuiltTicino) {
        var i = (function () {
            for (var j = 0; j < args.length; j++) {
                if (args[j][0] !== '-') {
                    return j;
                }
            }
            return -1;
        })();
        if (i > -1) {
            args.splice(i, 1);
        }
    }
    // Finally, any extra arguments in the 'argv' file should be prepended
    if (fs.existsSync(path.join(exports.appRoot, 'argv'))) {
        var extraargs = JSON.parse(fs.readFileSync(path.join(exports.appRoot, 'argv'), 'utf8'));
        args = extraargs.concat(args);
    }
    var opts = parseOpts(args);
    var gotoLineMode = !!opts['g'] || !!opts['goto'];
    return {
        pathArguments: parsePathArguments(args, gotoLineMode),
        programStart: parseNumber(args, '--timestamp', 0),
        workers: parseNumber(args, '--workers', -1),
        enablePerformance: !!opts['p'],
        enableSelfhost: !!opts['m'],
        verboseLogging: !!opts['verbose'],
        firstrun: !!opts['squirrel-firstrun'],
        openNewWindow: !!opts['n'] || !!opts['new-window'],
        openInSameWindow: !!opts['r'] || !!opts['reuse-window'],
        gotoLineMode: gotoLineMode
    };
}
function getAppRoot() {
    // Built Ticino
    if (exports.isBuiltTicino) {
        return __dirname;
    }
    // In-Development Ticino
    return path.resolve(path.join(__dirname, '..', '..'));
}
function getRunningInstanceHandle() {
    var handleName;
    if (exports.flavor) {
        handleName = exports.flavor;
    }
    else {
        handleName = app.getName();
        if (!exports.isBuiltTicino) {
            handleName += '-dev';
        }
    }
    // Support to run VS Code multiple times as different user
    // by making the socket unique over the logged in user
    var userId = uniqueUserId();
    if (userId) {
        handleName += ('-' + userId);
    }
    // Windows: use named pipe
    if (process.platform === 'win32') {
        return '\\\\.\\pipe\\' + handleName + '-sock';
    }
    // Mac/Unix: use socket file
    return path.join(os.tmpdir(), handleName + '.sock');
}
function uniqueUserId() {
    var username;
    if (exports.isWindows) {
        username = process.env.USERNAME;
    }
    else {
        username = process.env.USER;
    }
    if (!username) {
        return ''; // fail gracefully if there is no user name
    }
    // use sha256 to ensure the userid value can be used in filenames and are unique
    return crypto.createHash('sha256').update(username).digest('hex').substr(0, 6);
}
function parseOpts(argv) {
    return argv.filter(function (a) { return /^-/.test(a); }).map(function (a) { return a.replace(/^-*/, ''); }).reduce(function (r, a) {
        r[a] = true;
        return r;
    }, {});
}
function parsePathArguments(argv, gotoLineMode) {
    return arrays.distinct(argv.filter(function (a) { return !(/^-/.test(a)); }).map(function (arg) {
        var pathCandidate = arg;
        var parsedPath;
        if (gotoLineMode) {
            parsedPath = paths.parseLineAndColumnAware(arg);
            pathCandidate = parsedPath.path;
        }
        var realPath;
        try {
            realPath = fs.realpathSync(pathCandidate);
        }
        catch (error) {
            // in case of an error, assume the user wants to create this file
            // if the path is relative, we join it to the cwd
            realPath = path.normalize(path.isAbsolute(pathCandidate) ? pathCandidate : path.join(process.cwd(), pathCandidate));
        }
        if (gotoLineMode) {
            parsedPath.path = realPath;
            return paths.toLineAndColumnPath(parsedPath);
        }
        return realPath;
    }), function (element) {
        return element && (exports.isWindows || exports.isMac) ? element.toLowerCase() : element; // only linux is case sensitive on the fs
    });
}
function parseNumber(argv, key, defaultValue) {
    var value;
    for (var i = 0; i < argv.length; i++) {
        if (argv[i] === key && argv.length > i + 1) {
            value = Number(argv[i + 1]) || void 0;
            break;
        }
    }
    return objects.isNumber(value) ? value : defaultValue;
}
function getPlatformIdentifier() {
    if (process.platform === 'linux') {
        return "linux-" + process.arch;
    }
    return process.platform;
}
exports.getPlatformIdentifier = getPlatformIdentifier;
