/*!---------------------------------------------------------
* Copyright (C) Microsoft Corporation. All rights reserved.
*--------------------------------------------------------*/
/// <reference path="./declare/node.d.ts" />
/// <reference path="./declare/atom-browser.d.ts" />
'use strict';
// Perf measurements
global.ticinoStart = new Date().getTime();
try {
    process.env.VSCODE_CWD && process.chdir(process.env.VSCODE_CWD);
}
catch (err) {
}
// We handle uncaught exceptions here to prevent atom shell from opening a dialog to the user
process.on('uncaughtException', function (err) {
    if (err) {
        // take only the message and stack property
        var friendlyError = {
            message: err.message,
            stack: err.stack
        };
        // handle on client side
        windows.manager.sendToFocused('ticino:reportError', JSON.stringify(friendlyError));
    }
    console.error('[uncaught exception in main]: ', err);
    if (err.stack) {
        console.error(err.stack);
    }
});
// Do not move this, it should be loaded first!
var env = require('./env');
// Used by sub processes to communicate back to the main instance
process.env['VSCODE_IPC_HOOK'] = env.runningInstanceHandle;
var app = require('app');
var fs = require('fs');
var path = require('path');
var cp = require('child_process');
var nls = require('./nls');
var windows = require('./windows');
var plugins = require('./servants');
var menu = require('./menu');
var settings = require('./settings');
var squirrelwin32 = require('./win32/squirrel.win32');
var um = require('./update-manager');
var storage = require('./storage');
var events = require('./events');
var ipcbus = require('./ipcbus');
var git = require('./git/index');
var UpdateManager = um.Instance;
// Remember the app ready state, in case someone attaches the listener after the event has fired 
var isAppReady = false;
app.once('ready', function () { return isAppReady = true; });
// Mac: when someone drops a file to the not-yet running VSCode, the open-file event fires even before
// the app-ready event. We listen very early for open-file and remember this upon startup as path to open.
var macOpenFiles = [];
app.on('open-file', function (event, path) {
    macOpenFiles.push(path);
});
function onAppReady(cb) {
    if (isAppReady) {
        cb();
    }
    else {
        app.once('ready', cb);
    }
}
var Ticino = (function () {
    function Ticino() {
        var _this = this;
        // IPC bus
        this.ipcbus = new events.EventEmitter();
        // Setup IPC listeners
        this.setupIPCListeners();
        // Connect IPC
        this.connect();
        // Dispose on app quit
        app.on('will-quit', function () { return _this.dispose(); });
        // Load settings
        settings.manager.load();
        // Propagate to clients
        windows.manager.ready();
        plugins.manager.ready();
        // Open our first window
        if (env.cliArgs.openNewWindow) {
            windows.manager.open({ cli: env.cliArgs, forceNewWindow: true, forceEmpty: true }); // empty window if "-n" was used
        }
        else if (macOpenFiles.length && (!env.cliArgs.pathArguments || !env.cliArgs.pathArguments.length)) {
            windows.manager.open({ cli: env.cliArgs, pathsToOpen: macOpenFiles }); // mac: open-file event received on startup
        }
        else {
            windows.manager.open({ cli: env.cliArgs }); // default: read paths from cli
        }
        // Install Menu
        menu.manager.install();
        // Install Tasks
        if (env.isWindows) {
            app.setUserTasks([
                {
                    title: nls.localize('newWindow', "New Window"),
                    program: process.execPath,
                    arguments: env.isBuiltTicino ? '-n' : process.argv[1] + ' -n',
                    iconPath: process.execPath,
                    iconIndex: 0
                }
            ]);
        }
        // Configure git
        git.configure(this.ipcbus);
    }
    Ticino.startup = function () {
        if (!!Ticino.INSTANCE) {
            throw new Error('Can only ever have one instance at the same time');
        }
        Ticino.INSTANCE = new Ticino();
    };
    Ticino.prototype.setupIPCListeners = function () {
        this.ipcbus.addListener('start', function (data) {
            env.log('Received data from other instance', data);
            if (data.pathArguments.length === 0 && data.openNewWindow) {
                windows.manager.open({ cli: data, forceNewWindow: true, forceEmpty: true });
            }
            else if (data.pathArguments.length === 0) {
                windows.manager.focusLastActive(data);
            }
            else {
                windows.manager.open({ cli: data, forceNewWindow: !data.openInSameWindow });
            }
        });
    };
    Ticino.prototype.connect = function () {
        this.disconnect();
        this.ipcserver = ipcbus.publish(env.runningInstanceHandle, this.ipcbus);
        // This can happen when multiple apps fight over the same connection
        this.ipcserver.on('error', function (error) {
            console.error('Terminating because running instance listener failed:', error.toString());
            setTimeout(function () { return app.terminate(); }, 0); // we cannot allow multiple apps, so we quit
        });
    };
    Ticino.prototype.disconnect = function () {
        if (this.ipcserver) {
            this.ipcserver.close();
            this.ipcserver = null;
        }
        if (!env.isWindows && fs.existsSync(env.runningInstanceHandle)) {
            try {
                return fs.unlinkSync(env.runningInstanceHandle);
            }
            catch (e) {
                if (e.code !== 'ENOENT') {
                    env.log('Fatal error deleting running instance handle', e);
                    throw e;
                }
            }
        }
    };
    Ticino.prototype.dispose = function () {
        env.log('App#dispose: deleting running instance handle');
        this.disconnect();
    };
    return Ticino;
})();
function getUpdateFeedUrl() {
    if (!env.updateInfo || !env.updateInfo.baseUrl) {
        return null;
    }
    var baseUrl = env.updateInfo.baseUrl;
    var commit = env.commit;
    var channel = storage.getItem('updateChannel');
    if (!channel) {
        channel = 'stable';
        storage.setItem('updateChannel', channel);
    }
    if (process.platform !== 'win32') {
        return "" + baseUrl + "/api/update/" + env.getPlatformIdentifier() + "/" + channel + "/" + commit;
    }
    // We get the name of the parent dir of Ticino.exe. The name of this dir
    // contains the NuGet version which we use for the updates. It is not the
    // actual Ticino version, but a mere version to make Squirrel.Windows work.
    var parentDirName = path.basename(path.dirname(process.execPath));
    var match = /-(\d+\.\d+\.\d+)$/.exec(parentDirName);
    if (!match) {
        return null;
    }
    var nugetVersion = match[1];
    return "" + baseUrl + "/api/update/win32/" + channel + "/" + commit + "/" + nugetVersion;
}
function main() {
    global.programStart = env.cliArgs.programStart;
    // Setup update
    var updateFeedUrl = getUpdateFeedUrl();
    if (updateFeedUrl) {
        UpdateManager.initialize(updateFeedUrl);
        // Check for updates on startup after 30 seconds
        var timer = setTimeout(function () { return UpdateManager.checkForUpdates(); }, 30 * 1000);
        // Clear timer when checking for update
        UpdateManager.on('error', function (error, message) { return console.error(error, message); });
        // Clear timer when checking for update
        UpdateManager.on('checking-for-update', function () { return clearTimeout(timer); });
        // If update not found, try again in 10 minutes
        UpdateManager.on('update-not-available', function () {
            timer = setTimeout(function () { return UpdateManager.checkForUpdates(); }, 10 * 60 * 1000);
        });
    }
    env.log('### Ticino main.js ###');
    env.log(env.appRoot, env.cliArgs);
    // Ready for creating browser windows
    onAppReady(function () {
        // On Mac/Unix we can rely on the server handle to be deleted if the instance is not running
        if (!env.isWindows && !fs.existsSync(env.runningInstanceHandle)) {
            env.log('Mac/Linux: starting ticino because handle does not exist');
            Ticino.startup();
        }
        else {
            var didErr = false;
            var onError = function (error) {
                if (didErr) {
                    return;
                }
                didErr = true;
                env.log('Starting ticino because connection failed', error);
                Ticino.startup();
            };
            var socket = ipcbus.connect(env.runningInstanceHandle, function (err, bus) {
                if (err) {
                    return onError(err);
                }
                if (didErr) {
                    return;
                }
                env.log('Sending env to running instance and terminating');
                bus.emit('start', env.cliArgs);
                socket.end();
                setTimeout(function () { return app.terminate(); }, 0); // terminate later to prevent bad things from happen
            });
            socket.on('error', onError);
        }
    });
}
// On Darwin, the PATH environment will not be correctly set when double clicking
// the application. Need to fix it up before anything starts up.
function fixUnixEnvironment(cb) {
    var didReturn = false;
    var done = function () {
        if (didReturn) {
            return;
        }
        didReturn = true;
        cb();
    };
    var child = cp.spawn(process.env.SHELL, ['-ilc', 'env'], {
        detached: true,
        stdio: ['ignore', 'pipe', process.stderr],
    });
    child.stdout.setEncoding('utf8');
    child.on('error', done);
    var buffer = '';
    child.stdout.on('data', function (d) {
        buffer += d;
    });
    child.on('close', function (code, signal) {
        if (code !== 0) {
            return done();
        }
        var hash = Object.create(null);
        buffer.split('\n').forEach(function (line) {
            var pos = line.indexOf('=');
            if (pos > 0) {
                var key = line.substring(0, pos);
                var value = line.substring(pos + 1);
                if (!key || hash[key]) {
                    return;
                }
                hash[key] = true;
                process.env[key] = value;
            }
        });
        done();
    });
}
if (env.isMac || env.isLinux) {
    fixUnixEnvironment(main);
}
else if (env.isWindows && squirrelwin32.handleArgv()) {
}
else {
    main();
}
