/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
/// <reference path='../../declare/node.d.ts' />
'use strict';
define(["require", "exports", 'fs', 'path', '../../lib/flow', '../../lib/types', '../../lib/uri', '../../lib/glob', '../../lib/strings', '../../lib/extfs'], function (require, exports, fs, npath, flow, types, uri, glob, strings, extfs) {
    var CamelCaseExp = (function () {
        function CamelCaseExp(pattern) {
            this.pattern = pattern.toLowerCase();
        }
        CamelCaseExp.prototype.test = function (value) {
            if (value.length === 0) {
                return false;
            }
            var pattern = this.pattern.toLowerCase();
            var result;
            var i = 0;
            while (i < value.length && !(result = matches(pattern, value, 0, i))) {
                i = nextAnchor(value, i + 1);
            }
            return result;
        };
        return CamelCaseExp;
    })();
    exports.CamelCaseExp = CamelCaseExp;
    function isUpper(c) {
        var code = c.charCodeAt(0);
        return 65 <= code && code <= 90;
    }
    function isNumber(c) {
        var code = c.charCodeAt(0);
        return 48 <= code && code <= 57;
    }
    function nextAnchor(value, start) {
        var c;
        for (var i = start; i < value.length; i++) {
            c = value[i];
            if (isUpper(c) || isNumber(c)) {
                return i;
            }
        }
        return value.length;
    }
    function matches(pattern, value, patternIndex, valueIndex) {
        if (patternIndex === pattern.length) {
            return true;
        }
        if (valueIndex === value.length) {
            return false;
        }
        if (pattern[patternIndex] !== value[valueIndex].toLowerCase()) {
            return false;
        }
        var nextUpperIndex = valueIndex + 1;
        var result = matches(pattern, value, patternIndex + 1, valueIndex + 1);
        while (!result && (nextUpperIndex = nextAnchor(value, nextUpperIndex)) < value.length) {
            result = matches(pattern, value, patternIndex + 1, nextUpperIndex);
            nextUpperIndex++;
        }
        return result;
    }
    function isCamelCasePattern(pattern) {
        return (/^\w[\w.]*$/).test(pattern);
    }
    var FilePatterns = (function () {
        function FilePatterns(expressions) {
            this.expressions = [];
            for (var i = 0; i < expressions.length; i++) {
                var expression = expressions[i];
                var exp;
                // Match all
                if (!expression.pattern) {
                    exp = { test: function () { return true; } };
                }
                else if (expression.isRegExp) {
                    try {
                        exp = new RegExp(expression.pattern, 'i');
                    }
                    catch (e) {
                        if (e instanceof SyntaxError) {
                            exp = { test: function () { return true; } };
                        }
                        else {
                            throw e;
                        }
                    }
                }
                else if (isCamelCasePattern(expression.pattern)) {
                    exp = new CamelCaseExp(expression.pattern);
                }
                else {
                    if (expression.pattern.charCodeAt(0) === FilePatterns._dot) {
                        expression.pattern = '*' + expression.pattern; // convert a .<something> to a *.<something> query
                    }
                    // escape to regular expressions
                    expression.pattern = strings.anchorPattern(strings.convertSimple2RegExpPattern(expression.pattern), true, false);
                    exp = new RegExp(expression.pattern, 'i');
                }
                this.expressions.push(exp);
            }
        }
        FilePatterns.hasPatterns = function (expressions) {
            return expressions && expressions.length > 0 && expressions.some(function (e) { return !!e.pattern; });
        };
        FilePatterns.prototype.test = function (value) {
            for (var i = 0; i < this.expressions.length; i++) {
                var exp = this.expressions[i];
                if (exp.test(value)) {
                    return exp;
                }
            }
            return null;
        };
        FilePatterns._dot = '.'.charCodeAt(0);
        return FilePatterns;
    })();
    exports.FilePatterns = FilePatterns;
    var FileWalker = (function () {
        function FileWalker(config) {
            this.config = config;
            this.patterns = FilePatterns.hasPatterns(config.filePatterns) && new FilePatterns(config.filePatterns);
            this.excludePattern = config.excludePattern;
            this.includePattern = config.includePattern;
            this.maxResults = config.maxResults || null;
            this.walkedPaths = Object.create(null);
        }
        FileWalker.prototype.resetState = function () {
            this.walkedPaths = Object.create(null); // reset
            this.resultCount = 0;
            this.isLimitHit = false;
        };
        FileWalker.prototype.cancel = function () {
            this.isCanceled = true;
        };
        FileWalker.prototype.walk = function (rootResources, onResult, done) {
            var _this = this;
            // Reset state
            this.resetState();
            // For each source
            flow.parallel(rootResources, function (rootResourceStr, perEntryCallback) {
                var rootResource = uri.parse(rootResourceStr);
                var absolutePath = rootResource.fsPath;
                // Try to Read as folder
                fs.readdir(absolutePath, function (error, files) {
                    if (_this.isCanceled || _this.isLimitHit) {
                        return perEntryCallback(null, null);
                    }
                    // Handle Directory
                    if (!error) {
                        return _this.doWalk(absolutePath, '', files, onResult, perEntryCallback);
                    }
                    // Not a folder - deal with file result then
                    if (error.code === FileWalker.ENOTDIR && !_this.isCanceled && !_this.isLimitHit) {
                        // Check exclude pattern
                        if (glob.match(_this.excludePattern, absolutePath)) {
                            return perEntryCallback(null, null);
                        }
                        // Check for match on file pattern and include pattern
                        if ((!_this.patterns || _this.patterns.test(npath.basename(absolutePath))) && (!_this.includePattern || glob.match(_this.includePattern, absolutePath))) {
                            _this.resultCount++;
                            if (_this.maxResults && _this.resultCount > _this.maxResults) {
                                _this.isLimitHit = true;
                            }
                            if (!_this.isLimitHit) {
                                onResult({
                                    resource: rootResource.toString()
                                });
                            }
                        }
                    }
                    // Unwind
                    return perEntryCallback(null, null);
                });
            }, function (err, result) {
                done(err ? err[0] : null, _this.isLimitHit);
            });
        };
        FileWalker.prototype.doWalk = function (absolutePath, relativeParentPath, files, onResult, done) {
            var _this = this;
            // Execute tasks on each file in parallel to optimize throughput
            flow.parallel(files, function (file, clb) {
                // Check canceled
                if (_this.isCanceled || _this.isLimitHit) {
                    return clb(null);
                }
                // If the user searches for the exact file name, we adjust the glob matching
                // to ignore filtering by siblings because the user seems to know what she
                // is searching for and we want to include the result in that case anyway
                var siblings = files;
                if (_this.config.filePatterns && _this.config.filePatterns.length === 1 && _this.config.filePatterns[0].pattern === file) {
                    siblings = [];
                }
                // Check exclude pattern
                var relativeFilePath = strings.trim([relativeParentPath, file].join('/'), '/');
                if (glob.match(_this.excludePattern, relativeFilePath, siblings)) {
                    return clb(null);
                }
                // Try to read dir
                var currentPath = npath.join(absolutePath, file);
                fs.readdir(currentPath, function (error, children) {
                    // Handle directory
                    if (!error) {
                        // to really prevent loops with links we need to resolve the real path of them
                        return extfs.realPathLink(currentPath, function (error, realpath) {
                            if (error) {
                                return clb(null); // ignore errors
                            }
                            if (_this.walkedPaths[realpath]) {
                                return clb(null); // escape when there are cycles (can happen with symlinks)
                            }
                            else {
                                _this.walkedPaths[realpath] = true; // remember as walked
                            }
                            // Continue walking
                            _this.doWalk(currentPath, relativeFilePath, children, onResult, clb);
                        });
                    }
                    // Handle file if we are not canceled and have not hit the limit yet
                    if (error.code === FileWalker.ENOTDIR && !_this.isCanceled && !_this.isLimitHit) {
                        // Check for match on file pattern and include pattern
                        if ((!_this.patterns || _this.patterns.test(file)) && (!_this.includePattern || glob.match(_this.includePattern, relativeFilePath, children))) {
                            _this.resultCount++;
                            if (_this.maxResults && _this.resultCount > _this.maxResults) {
                                _this.isLimitHit = true;
                            }
                            if (!_this.isLimitHit) {
                                onResult({
                                    resource: uri.file(currentPath).toString()
                                });
                            }
                        }
                    }
                    // Unwind
                    return clb(null);
                });
            }, function (error) {
                if (error) {
                    error = types.coalesce(error); // find any error by removing null values first
                }
                return done(error && error.length > 0 ? error[0] : null, null);
            });
        };
        FileWalker.ENOTDIR = 'ENOTDIR';
        return FileWalker;
    })();
    exports.FileWalker = FileWalker;
    var Engine = (function () {
        function Engine(config) {
            this.rootResources = config.rootResources;
            this.walker = new FileWalker(config);
        }
        Engine.prototype.search = function (onResult, onProgress, done) {
            this.walker.walk(this.rootResources, onResult, done);
        };
        Engine.prototype.cancel = function () {
            this.walker.cancel();
        };
        return Engine;
    })();
    exports.Engine = Engine;
});
