/*!---------------------------------------------------------
* Copyright (C) Microsoft Corporation. All rights reserved.
*--------------------------------------------------------*/
/// <reference path="./declare/node.d.ts" />
/// <reference path="./declare/atom-browser.d.ts" />
'use strict';
var events = require('events');
var path = require('path');
var fs = require('fs');
var BrowserWindow = require('browser-window');
var Dialog = require('dialog');
var app = require('app');
var ipc = require('ipc');
var env = require('./env');
var window = require('./window');
var paths = require('./paths');
var arrays = require('./arrays');
var objects = require('./objects');
var storage = require('./storage');
var settings = require('./settings');
var nls = require('./nls');
var um = require('./update-manager');
var UpdateManager = um.Instance;
var eventEmitter = new events.EventEmitter();
var EventTypes = {
    OPEN: 'open',
    CLOSE: 'close',
    QUIT: 'quit'
};
function onOpen(clb) {
    eventEmitter.addListener(EventTypes.OPEN, clb);
    return function () { return eventEmitter.removeListener(EventTypes.OPEN, clb); };
}
exports.onOpen = onOpen;
function onClose(clb) {
    eventEmitter.addListener(EventTypes.CLOSE, clb);
    return function () { return eventEmitter.removeListener(EventTypes.CLOSE, clb); };
}
exports.onClose = onClose;
function onQuit(clb) {
    eventEmitter.addListener(EventTypes.QUIT, clb);
    return function () { return eventEmitter.removeListener(EventTypes.QUIT, clb); };
}
exports.onQuit = onQuit;
var WindowError;
(function (WindowError) {
    WindowError[WindowError["UNRESPONSIVE"] = 0] = "UNRESPONSIVE";
    WindowError[WindowError["CRASHED"] = 1] = "CRASHED";
})(WindowError || (WindowError = {}));
var WindowsManager = (function () {
    function WindowsManager() {
    }
    WindowsManager.prototype.ready = function () {
        this.registerListeners();
    };
    WindowsManager.prototype.registerListeners = function () {
        var _this = this;
        settings.onChange(function (newSettings) {
            _this.sendToAll('ticino:optionsChange', JSON.stringify({ globalSettings: newSettings }));
        });
        app.on('window-all-closed', function () {
            env.log('App#window-all-closed');
            if (process.platform !== 'darwin') {
                app.quit();
            }
        });
        app.on('will-quit', function () {
            eventEmitter.emit(EventTypes.QUIT);
        });
        app.on('activate-with-no-open-windows', function () {
            env.log('App#activate-with-no-open-windows');
            // We want to open the previously opened folder, so we dont pass on the path argument
            var cliArgWithoutPath = objects.clone(env.cliArgs);
            cliArgWithoutPath.pathArguments = [];
            exports.manager.open({ cli: cliArgWithoutPath });
        });
        var macOpenFiles = [];
        var runningTimeout = null;
        app.on('open-file', function (event, path) {
            env.log('App#open-file: ', path);
            event.preventDefault();
            // Keep in array because more might come!
            macOpenFiles.push(path);
            // Clear previous handler if any
            if (runningTimeout !== null) {
                clearTimeout(runningTimeout);
                runningTimeout = null;
            }
            // Handle paths delayed in case more are coming!
            runningTimeout = setTimeout(function () {
                exports.manager.open({ cli: env.cliArgs, pathsToOpen: macOpenFiles, forceNewWindow: true /* dropping on the dock should force open in a new window */ });
                macOpenFiles = [];
                runningTimeout = null;
            }, 100);
        });
        ipc.on('ticino:windowOpen', function (event, paths, forceNewWindow) {
            env.log('IPC#ticino-windowOpen: ', paths);
            if (paths && paths.length) {
                exports.manager.open({ cli: env.cliArgs, pathsToOpen: paths, forceNewWindow: forceNewWindow });
            }
        });
        ipc.on('ticino:openFilePicker', function (event) {
            env.log('IPC#ticino-openFilePicker');
            exports.manager.openFilePicker();
        });
        ipc.on('ticino:openFolderPicker', function (event) {
            env.log('IPC#ticino-openFolderPicker');
            exports.manager.openFolderPicker();
        });
        ipc.on('ticino:closeFolder', function (event, windowId) {
            env.log('IPC#ticino-closeFolder');
            var win = _this.getWindowById(windowId);
            if (win) {
                exports.manager.open({ cli: env.cliArgs, forceEmpty: true, windowToUse: win });
            }
        });
        ipc.on('ticino:openNewWindow', function (event) {
            env.log('IPC#ticino-openNewWindow');
            exports.manager.openNewWindow();
        });
        ipc.on('ticino:openFileFolderPicker', function (event) {
            env.log('IPC#ticino-openFileFolderPicker');
            exports.manager.openFolderPicker();
        });
        ipc.on('ticino:unloadWindow', function (event, windowId) {
            env.log('IPC#ticino:unloadWindow');
            // make sure to not run this within the callback to prevent issues
            setTimeout(function () {
                var win = _this.getWindowById(windowId);
                if (win) {
                    win.unload();
                }
            }, 0);
        });
        ipc.on('ticino:reloadWindow', function (event, windowId) {
            env.log('IPC#ticino:reloadWindow');
            var win = _this.getWindowById(windowId);
            if (win) {
                win.load(_this.toConfiguration(env.cliArgs, win.openedWorkspacePath));
            }
        });
        UpdateManager.on('update-downloaded', function (update) {
            _this.sendToAll('ticino:update-downloaded', JSON.stringify({
                releaseNotes: update.releaseNotes,
                version: update.version,
                date: update.date
            }));
        });
        ipc.on('ticino:update-apply', function (event) {
            env.log('IPC#ticino:update-apply');
            if (UpdateManager.availableUpdate) {
                UpdateManager.availableUpdate.quitAndUpdate();
            }
        });
        UpdateManager.on('update-not-available', function (explicit) {
            if (explicit) {
                _this.sendToFocused('ticino:update-not-available', '');
            }
        });
    };
    WindowsManager.prototype.open = function (openConfig) {
        var _this = this;
        var iPathsToOpen;
        // Find paths from provided paths if any
        if (openConfig.pathsToOpen && openConfig.pathsToOpen.length > 0) {
            iPathsToOpen = openConfig.pathsToOpen.map(function (pathToOpen) {
                var iPath = _this.toIPath(pathToOpen, false, openConfig.cli && openConfig.cli.gotoLineMode);
                // Warn if the requested path to open does not exist
                if (!iPath) {
                    Dialog.showMessageBox({
                        type: 'warning',
                        buttons: [nls.localize('ok', "OK")],
                        title: nls.localize('pathNotExistTitle', "Path does not exist"),
                        detail: nls.localize('pathNotExistDetail', "The path '{0}' does not seem to exist anymore on disk.", pathToOpen)
                    });
                }
                return iPath;
            });
            // get rid of nulls
            iPathsToOpen = arrays.coalesce(iPathsToOpen);
            if (iPathsToOpen.length === 0) {
                return false; // indicate to outside that open failed
            }
        }
        else {
            var ignoreFileNotFound = openConfig.cli.pathArguments.length > 0; // we assume the user wants to create this file from command line
            iPathsToOpen = openConfig.forceEmpty ? [{}] : this.envToIPath(openConfig.cli, ignoreFileNotFound);
        }
        var filesToOpen = iPathsToOpen.filter(function (iPath) { return !!iPath.filePath && !iPath.createFilePath; });
        var filesToCreate = iPathsToOpen.filter(function (iPath) { return !!iPath.filePath && iPath.createFilePath; });
        var foldersToOpen = iPathsToOpen.filter(function (iPath) { return iPath.workspacePath && !iPath.filePath; });
        var emptyToOpen = iPathsToOpen.filter(function (iPath) { return !iPath.workspacePath && !iPath.filePath; });
        var configuration;
        // Handle files to open or to create when we dont open a folder
        if (!foldersToOpen.length && (filesToOpen.length > 0 || filesToCreate.length > 0)) {
            // Open Files in last instance if any and we are not to force open in a new window
            var lastActiveWindow = this.getLastActiveWindow();
            if (!openConfig.forceNewWindow && lastActiveWindow && lastActiveWindow.isLoaded) {
                lastActiveWindow.restore(true);
                lastActiveWindow.win.webContents.send('ticino:openFiles', {
                    filesToOpen: filesToOpen,
                    filesToCreate: filesToCreate
                });
            }
            else {
                configuration = this.toConfiguration(openConfig.cli, null, filesToOpen, filesToCreate);
                this.openInBrowserWindow(configuration, true);
                openConfig.forceNewWindow = true; // any other folders to open must open in new window then
            }
        }
        // Handle folders to open
        if (foldersToOpen.length > 0) {
            // Check for existing instances
            var windowsOnWorkspacePath = arrays.coalesce(foldersToOpen.map(function (iPath) { return _this.findWindow(iPath.workspacePath); }));
            if (windowsOnWorkspacePath.length > 0) {
                windowsOnWorkspacePath[0].restore(true); // just focus one of them
                windowsOnWorkspacePath[0].win.webContents.send('ticino:openFiles', {
                    filesToOpen: filesToOpen,
                    filesToCreate: filesToCreate
                });
                // Reset these because we handled them
                filesToOpen = [];
                filesToCreate = [];
                openConfig.forceNewWindow = true; // any other folders to open must open in new window then
            }
            // Open remaining ones
            foldersToOpen.forEach(function (folderToOpen) {
                if (windowsOnWorkspacePath.some(function (win) { return win.openedWorkspacePath === folderToOpen.workspacePath; })) {
                    return; // ignore folders that are already open
                }
                configuration = _this.toConfiguration(openConfig.cli, folderToOpen.workspacePath, filesToOpen, filesToCreate);
                _this.openInBrowserWindow(configuration, openConfig.forceNewWindow, openConfig.forceNewWindow ? void 0 : openConfig.windowToUse);
                // Reset these because we handled them
                filesToOpen = [];
                filesToCreate = [];
                openConfig.forceNewWindow = true; // any other folders to open must open in new window then
            });
        }
        // Handle empty
        if (emptyToOpen.length > 0) {
            emptyToOpen.forEach(function () {
                var configuration = _this.toConfiguration(openConfig.cli);
                _this.openInBrowserWindow(configuration, openConfig.forceNewWindow, openConfig.forceNewWindow ? void 0 : openConfig.windowToUse);
                openConfig.forceNewWindow = true; // any other folders to open must open in new window then
            });
        }
        // Remember in recent document list
        iPathsToOpen.forEach(function (iPath) {
            if (iPath.filePath || iPath.workspacePath) {
                app.addRecentDocument(iPath.filePath || iPath.workspacePath);
            }
        });
        // Emit events
        iPathsToOpen.forEach(function (iPath) { return eventEmitter.emit(EventTypes.OPEN, iPath); });
        return true;
    };
    WindowsManager.prototype.toConfiguration = function (cli, workspacePath, filesToOpen, filesToCreate) {
        var configuration = objects.mixin({}, cli);
        configuration.workspacePath = workspacePath;
        configuration.filesToOpen = filesToOpen;
        configuration.filesToCreate = filesToCreate;
        configuration.appNameLong = env.appNameLong;
        configuration.appNameShort = env.appNameShort;
        configuration.appRoot = env.appRoot;
        configuration.version = env.version;
        configuration.commitHash = env.commit;
        configuration.appSettingsHome = env.appSettingsHome;
        configuration.appSettingsPath = env.appSettingsPath;
        configuration.appKeybindingsPath = env.appKeybindingsPath;
        configuration.appLaunchSettingsPath = env.appLaunchSettingsPath;
        configuration.isBuiltTicino = env.isBuiltTicino;
        configuration.welcomePath = env.welcomePath;
        configuration.autoSaveDelay = storage.getItem(WindowsManager.autoSaveDelayStorageKey) || -1;
        return configuration;
    };
    WindowsManager.prototype.toIPath = function (anyPath, ignoreFileNotFound, gotoLineMode) {
        if (!anyPath) {
            return null;
        }
        var parsedPath;
        if (gotoLineMode) {
            parsedPath = paths.parseLineAndColumnAware(anyPath);
            anyPath = parsedPath.path;
        }
        var candidate = path.normalize(anyPath);
        try {
            var candidateStat = fs.statSync(candidate);
            if (candidateStat) {
                return candidateStat.isFile() ? {
                    filePath: candidate,
                    lineNumber: gotoLineMode ? parsedPath.line : void 0,
                    columnNumber: gotoLineMode ? parsedPath.column : void 0
                } : { workspacePath: candidate };
            }
        }
        catch (error) {
            if (ignoreFileNotFound) {
                return { filePath: candidate, createFilePath: true }; // assume this is a file that does not yet exist
            }
        }
        return null;
    };
    WindowsManager.prototype.envToIPath = function (cli, ignoreFileNotFound) {
        var _this = this;
        // Check for pass in candidate or last opened path
        var candidates = [];
        if (cli.pathArguments.length > 0) {
            candidates = cli.pathArguments;
        }
        else {
            var lastActivePath = storage.getItem(WindowsManager.lastActiveOpenedPathStorageKey);
            if (lastActivePath) {
                candidates.push(lastActivePath);
            }
        }
        var iPaths = candidates.map(function (candidate) { return _this.toIPath(candidate, ignoreFileNotFound, cli.gotoLineMode); }).filter(function (path) { return !!path; });
        if (iPaths.length > 0) {
            return iPaths;
        }
        // No path provided, return empty to open empty
        return [{}];
    };
    WindowsManager.prototype.openInBrowserWindow = function (configuration, forceNewWindow, windowToUse) {
        var _this = this;
        var ticinoWindow;
        if (!forceNewWindow) {
            ticinoWindow = windowToUse || this.getLastActiveWindow();
            if (ticinoWindow) {
                ticinoWindow.restore();
            }
        }
        if (!ticinoWindow) {
            ticinoWindow = new window.TicinoWindow(this.ensureNoOverlap(this.getNewWindowState()));
            WindowsManager.WINDOWS.push(ticinoWindow);
            // Window Events
            ticinoWindow.win.on('focus', function () { return _this.onWindowFocus(ticinoWindow); });
            ticinoWindow.win.webContents.on('crashed', function () { return _this.onWindowError(ticinoWindow.win, WindowError.CRASHED); });
            ticinoWindow.win.on('unresponsive', function () { return _this.onWindowError(ticinoWindow.win, WindowError.UNRESPONSIVE); });
            ticinoWindow.win.on('closed', function () { return _this.onWindowClosed(ticinoWindow); });
        }
        // Load it
        ticinoWindow.load(configuration);
        // Store last as active
        this.storeLastActivePath(configuration.workspacePath || (configuration.filesToOpen && configuration.filesToOpen[0] && configuration.filesToOpen[0].filePath));
    };
    WindowsManager.prototype.getNewWindowState = function () {
        var lastActive = this.getLastActiveWindow();
        if (!lastActive) {
            return null; // let the window restore itself if this is the first window
        }
        var screen = require('screen'); // Do not move, cannot import screen top level until app is ready
        // We want the new window to open on the same display that the last active one is in
        var displayToUse;
        var displays = screen.getAllDisplays();
        // Single Display
        if (displays.length === 1) {
            displayToUse = displays[0];
        }
        else {
            // on mac there is 1 menu per window so we need to use the monitor where the cursor currently is
            if (env.isMac) {
                var cursorPoint = screen.getCursorScreenPoint();
                displayToUse = screen.getDisplayNearestPoint(cursorPoint);
            }
            if (!displayToUse) {
                displayToUse = screen.getDisplayMatching(lastActive.getBounds());
            }
            if (!displayToUse) {
                return window.defaultWindowState(); // this should really not happen
            }
        }
        var defaultState = window.defaultWindowState();
        defaultState.x = displayToUse.bounds.x + (displayToUse.bounds.width / 2) - (defaultState.width / 2);
        defaultState.y = displayToUse.bounds.y + (displayToUse.bounds.height / 2) - (defaultState.height / 2);
        return defaultState;
    };
    WindowsManager.prototype.ensureNoOverlap = function (state) {
        if (!state || WindowsManager.WINDOWS.length === 0) {
            return state;
        }
        var existingWindowBounds = WindowsManager.WINDOWS.map(function (win) { return win.getBounds(); });
        while (existingWindowBounds.some(function (b) { return b.x === state.x || b.y === state.y; })) {
            state.x += 30;
            state.y += 30;
        }
        return state;
    };
    WindowsManager.prototype.openFilePicker = function () {
        var _this = this;
        this.getFileOrFolderPaths(false, function (paths) {
            if (paths && paths.length) {
                _this.open({ cli: env.cliArgs, pathsToOpen: paths });
            }
        });
    };
    WindowsManager.prototype.openFolderPicker = function () {
        var _this = this;
        this.getFileOrFolderPaths(true, function (paths) {
            if (paths && paths.length) {
                _this.open({ cli: env.cliArgs, pathsToOpen: paths });
            }
        });
    };
    WindowsManager.prototype.getFileOrFolderPaths = function (isFolder, clb) {
        var storageKey = isFolder ? WindowsManager.workingDirFolderPickerStorageKey : WindowsManager.workingDirFilePickerStorageKey;
        var workingDir = storage.getItem(storageKey);
        var focussedWindow = this.getFocusedWindow();
        var pickerProperties;
        if (env.isMac) {
            pickerProperties = ['multiSelections', 'openDirectory', 'openFile', 'createDirectory'];
        }
        else {
            pickerProperties = ['multiSelections', isFolder ? 'openDirectory' : 'openFile', 'createDirectory'];
        }
        Dialog.showOpenDialog(focussedWindow && focussedWindow.win, {
            defaultPath: workingDir,
            properties: pickerProperties
        }, function (paths) {
            if (paths && paths.length > 0) {
                // Remember path in storage for next time
                var selectedPath = paths[0];
                storage.setItem(storageKey, isFolder ? path.dirname(selectedPath) : selectedPath);
                // Return
                clb(paths);
            }
            else {
                clb(void (0));
            }
        });
    };
    WindowsManager.prototype.focusLastActive = function (cli) {
        var lastActive = this.getLastActiveWindow();
        if (lastActive) {
            lastActive.restore(true);
        }
        else {
            this.open({ cli: cli });
        }
    };
    WindowsManager.prototype.getLastActiveWindow = function () {
        if (WindowsManager.WINDOWS.length) {
            var lastFocussedDate = Math.max.apply(Math, WindowsManager.WINDOWS.map(function (w) { return w.lastFocusTime; }));
            var res = WindowsManager.WINDOWS.filter(function (w) { return w.lastFocusTime === lastFocussedDate; });
            if (res && res.length) {
                return res[0];
            }
        }
        return null;
    };
    WindowsManager.prototype.findWindow = function (workspacePath, filePath) {
        if (WindowsManager.WINDOWS.length) {
            // Sort the last active window to the front of the array of windows to test
            var windowsToTest = WindowsManager.WINDOWS.slice(0);
            var lastActiveWindow = this.getLastActiveWindow();
            if (lastActiveWindow) {
                windowsToTest.splice(windowsToTest.indexOf(lastActiveWindow), 1);
                windowsToTest.unshift(lastActiveWindow);
            }
            // Find it
            var res = windowsToTest.filter(function (w) {
                // match on workspace
                if (typeof w.openedWorkspacePath === 'string' && w.openedWorkspacePath === workspacePath) {
                    return true;
                }
                // match on file
                if (typeof w.openedFilePath === 'string' && w.openedFilePath === filePath) {
                    return true;
                }
                // match on file path
                if (typeof w.openedWorkspacePath === 'string' && filePath && paths.isEqualOrParent(filePath, w.openedWorkspacePath)) {
                    return true;
                }
                return false;
            });
            if (res && res.length) {
                return res[0];
            }
        }
        return null;
    };
    WindowsManager.prototype.openNewWindow = function () {
        this.open({ cli: env.cliArgs, forceNewWindow: true, forceEmpty: true });
    };
    WindowsManager.prototype.closeActive = function () {
        var ticinoWindow = this.getFocusedWindow();
        if (ticinoWindow) {
            ticinoWindow.win.close();
        }
    };
    WindowsManager.prototype.sendToFocused = function (channel, msg) {
        var focusedWindow = this.getFocusedWindow();
        if (focusedWindow && focusedWindow.isLoaded) {
            focusedWindow.win.webContents.send(channel, msg);
        }
    };
    WindowsManager.prototype.sendToLastActive = function (channel, msg) {
        var lastActiveWindow = this.getLastActiveWindow();
        if (lastActiveWindow && lastActiveWindow.isLoaded) {
            lastActiveWindow.win.webContents.send(channel, msg);
            return true;
        }
        return false;
    };
    WindowsManager.prototype.sendToAll = function (channel, msg) {
        WindowsManager.WINDOWS.forEach(function (window) {
            if (window.isLoaded) {
                window.win.webContents.send(channel, msg);
            }
        });
    };
    WindowsManager.prototype.getFocusedWindow = function () {
        var win = BrowserWindow.getFocusedWindow();
        if (win) {
            return this.getWindowById(win.id);
        }
        return null;
    };
    WindowsManager.prototype.getWindowById = function (windowId) {
        var res = WindowsManager.WINDOWS.filter(function (w) { return w.win.id === windowId; });
        if (res && res.length === 1) {
            return res[0];
        }
        return null;
    };
    WindowsManager.prototype.getWindowCount = function () {
        return WindowsManager.WINDOWS.length;
    };
    WindowsManager.prototype.onWindowError = function (win, error) {
        console.error(error === WindowError.CRASHED ? '[atom event]: render process crashed!' : '[atom event]: detected unresponsive');
        // Unresponsive
        if (error === WindowError.UNRESPONSIVE) {
            Dialog.showMessageBox(win, {
                type: 'warning',
                buttons: [nls.localize('exit', "Exit"), nls.localize('wait', "Keep Waiting")],
                message: nls.localize('appStalled', "{0} is no longer responding", env.appNameLong),
                detail: nls.localize('appStalledDetail', "Would you like to exit {0} or just keep waiting?", env.appNameLong)
            }, function (result) {
                if (result === 0) {
                    win.destroy(); // make sure to destroy the window as otherwise quit will just not do anything
                    app.quit();
                }
            });
        }
        else {
            Dialog.showMessageBox(win, {
                type: 'warning',
                buttons: [nls.localize('exit', "Exit")],
                message: nls.localize('appCrashed', "{0} has crashed", env.appNameLong),
                detail: nls.localize('appCrashedDetail', "We are sorry for the inconvenience! Please restart {0}.", env.appNameLong)
            }, function (result) {
                app.quit();
            });
        }
    };
    WindowsManager.prototype.onWindowFocus = function (win) {
        if (win.isLoadPending()) {
            return; // when the window is about to load another config, do not update the last active path because it might be stale
        }
        // whenever we get focus, make sure to update the last active opened path for future use
        this.storeLastActivePath(win.openedWorkspacePath || win.openedFilePath);
    };
    WindowsManager.prototype.storeLastActivePath = function (path) {
        if (path) {
            storage.setItem(WindowsManager.lastActiveOpenedPathStorageKey, path);
        }
        else {
            storage.removeItem(WindowsManager.lastActiveOpenedPathStorageKey);
        }
    };
    WindowsManager.prototype.onWindowClosed = function (win) {
        // Tell window
        win.dispose();
        // Remove from our list so that Atom can clean it up
        var index = WindowsManager.WINDOWS.indexOf(win);
        WindowsManager.WINDOWS.splice(index, 1);
        // Emit
        eventEmitter.emit(EventTypes.CLOSE, WindowsManager.WINDOWS.length);
    };
    WindowsManager.autoSaveDelayStorageKey = 'autoSaveDelay';
    WindowsManager.workingDirFilePickerStorageKey = 'filePickerWorkingDir';
    WindowsManager.workingDirFolderPickerStorageKey = 'folderPickerWorkingDir';
    WindowsManager.lastActiveOpenedPathStorageKey = 'lastActiveOpenedPath';
    WindowsManager.WINDOWS = [];
    return WindowsManager;
})();
exports.WindowsManager = WindowsManager;
exports.manager = new WindowsManager();
